# Form implementation generated from reading ui file 'ContestDlg.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_ContestDialog(object):
    def setupUi(self, ContestDialog):
        ContestDialog.setObjectName("ContestDialog")
        ContestDialog.resize(299, 547)
        self.verticalLayout = QtWidgets.QVBoxLayout(ContestDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(parent=ContestDialog)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label)
        self.contestComboBox = QtWidgets.QComboBox(parent=ContestDialog)
        self.contestComboBox.setObjectName("contestComboBox")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.contestComboBox)
        self.label_2 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_2)
        self.label_3 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_3)
        self.label_4 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_4)
        self.bandComboBox = QtWidgets.QComboBox(parent=ContestDialog)
        self.bandComboBox.setObjectName("bandComboBox")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.bandComboBox)
        self.fromDateEdit = QtWidgets.QDateEdit(parent=ContestDialog)
        self.fromDateEdit.setMinimumDate(QtCore.QDate(1900, 1, 1))
        self.fromDateEdit.setCurrentSection(QtWidgets.QDateTimeEdit.Section.YearSection)
        self.fromDateEdit.setCalendarPopup(True)
        self.fromDateEdit.setObjectName("fromDateEdit")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.fromDateEdit)
        self.toDateEdit = QtWidgets.QDateEdit(parent=ContestDialog)
        self.toDateEdit.setCalendarPopup(True)
        self.toDateEdit.setObjectName("toDateEdit")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.ItemRole.FieldRole, self.toDateEdit)
        self.line = QtWidgets.QFrame(parent=ContestDialog)
        self.line.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line.setObjectName("line")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.ItemRole.FieldRole, self.line)
        self.label_6 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_6.setObjectName("label_6")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_6)
        self.label_7 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_7.setObjectName("label_7")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_7)
        self.label_5 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(9, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_5)
        self.specificLabel = QtWidgets.QLabel(parent=ContestDialog)
        self.specificLabel.setObjectName("specificLabel")
        self.formLayout.setWidget(11, QtWidgets.QFormLayout.ItemRole.LabelRole, self.specificLabel)
        self.label_9 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_9.setObjectName("label_9")
        self.formLayout.setWidget(12, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_9)
        self.label_10 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_10.setObjectName("label_10")
        self.formLayout.setWidget(8, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_10)
        self.label_11 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_11.setObjectName("label_11")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_11)
        self.line_2 = QtWidgets.QFrame(parent=ContestDialog)
        self.line_2.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_2.setObjectName("line_2")
        self.formLayout.setWidget(13, QtWidgets.QFormLayout.ItemRole.FieldRole, self.line_2)
        self.label_13 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_13.setObjectName("label_13")
        self.formLayout.setWidget(14, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_13)
        self.label_12 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_12.setObjectName("label_12")
        self.formLayout.setWidget(15, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_12)
        self.callLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.callLineEdit.setObjectName("callLineEdit")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.ItemRole.FieldRole, self.callLineEdit)
        self.locatorLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.locatorLineEdit.setObjectName("locatorLineEdit")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.ItemRole.FieldRole, self.locatorLineEdit)
        self.nameLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.nameLineEdit.setObjectName("nameLineEdit")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.ItemRole.FieldRole, self.nameLineEdit)
        self.emailLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.emailLineEdit.setObjectName("emailLineEdit")
        self.formLayout.setWidget(8, QtWidgets.QFormLayout.ItemRole.FieldRole, self.emailLineEdit)
        self.specificLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.specificLineEdit.setObjectName("specificLineEdit")
        self.formLayout.setWidget(11, QtWidgets.QFormLayout.ItemRole.FieldRole, self.specificLineEdit)
        self.clubLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.clubLineEdit.setObjectName("clubLineEdit")
        self.formLayout.setWidget(12, QtWidgets.QFormLayout.ItemRole.FieldRole, self.clubLineEdit)
        self.powerComboBox = QtWidgets.QComboBox(parent=ContestDialog)
        self.powerComboBox.setObjectName("powerComboBox")
        self.formLayout.setWidget(14, QtWidgets.QFormLayout.ItemRole.FieldRole, self.powerComboBox)
        self.modeComboBox = QtWidgets.QComboBox(parent=ContestDialog)
        self.modeComboBox.setObjectName("modeComboBox")
        self.formLayout.setWidget(15, QtWidgets.QFormLayout.ItemRole.FieldRole, self.modeComboBox)
        self.streetLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.streetLineEdit.setObjectName("streetLineEdit")
        self.formLayout.setWidget(9, QtWidgets.QFormLayout.ItemRole.FieldRole, self.streetLineEdit)
        self.label_8 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_8.setObjectName("label_8")
        self.formLayout.setWidget(10, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_8)
        self.cityLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.cityLineEdit.setObjectName("cityLineEdit")
        self.formLayout.setWidget(10, QtWidgets.QFormLayout.ItemRole.FieldRole, self.cityLineEdit)
        self.line_3 = QtWidgets.QFrame(parent=ContestDialog)
        self.line_3.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_3.setObjectName("line_3")
        self.formLayout.setWidget(16, QtWidgets.QFormLayout.ItemRole.FieldRole, self.line_3)
        self.label_14 = QtWidgets.QLabel(parent=ContestDialog)
        self.label_14.setObjectName("label_14")
        self.formLayout.setWidget(17, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_14)
        self.soapPlainTextEdit = QtWidgets.QPlainTextEdit(parent=ContestDialog)
        self.soapPlainTextEdit.setMinimumSize(QtCore.QSize(0, 40))
        self.soapPlainTextEdit.setMaximumSize(QtCore.QSize(16777215, 60))
        self.soapPlainTextEdit.setObjectName("soapPlainTextEdit")
        self.formLayout.setWidget(17, QtWidgets.QFormLayout.ItemRole.FieldRole, self.soapPlainTextEdit)
        self.verticalLayout.addLayout(self.formLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.exportPathLineEdit = QtWidgets.QLineEdit(parent=ContestDialog)
        self.exportPathLineEdit.setObjectName("exportPathLineEdit")
        self.horizontalLayout_2.addWidget(self.exportPathLineEdit)
        self.choosePathToolButton = QtWidgets.QToolButton(parent=ContestDialog)
        self.choosePathToolButton.setObjectName("choosePathToolButton")
        self.horizontalLayout_2.addWidget(self.choosePathToolButton)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.exportPushButton = QtWidgets.QPushButton(parent=ContestDialog)
        self.exportPushButton.setEnabled(False)
        self.exportPushButton.setObjectName("exportPushButton")
        self.horizontalLayout.addWidget(self.exportPushButton)
        self.cancelPushButton = QtWidgets.QPushButton(parent=ContestDialog)
        self.cancelPushButton.setObjectName("cancelPushButton")
        self.horizontalLayout.addWidget(self.cancelPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(ContestDialog)
        self.exportPushButton.clicked.connect(ContestDialog.accept) # type: ignore
        self.cancelPushButton.clicked.connect(ContestDialog.reject) # type: ignore
        self.contestComboBox.currentTextChanged['QString'].connect(ContestDialog.contestChanged) # type: ignore
        self.fromDateEdit.dateChanged['QDate'].connect(ContestDialog.fromDateChanged) # type: ignore
        self.choosePathToolButton.clicked.connect(ContestDialog.choosePath) # type: ignore
        self.callLineEdit.textChanged['QString'].connect(ContestDialog.callChanged) # type: ignore
        self.locatorLineEdit.textChanged['QString'].connect(ContestDialog.locatorChanged) # type: ignore
        self.nameLineEdit.textChanged['QString'].connect(ContestDialog.nameChanged) # type: ignore
        self.cityLineEdit.textChanged['QString'].connect(ContestDialog.cityChanged) # type: ignore
        self.streetLineEdit.textChanged['QString'].connect(ContestDialog.streetChanged) # type: ignore
        self.emailLineEdit.textChanged['QString'].connect(ContestDialog.emailChanged) # type: ignore
        self.clubLineEdit.textChanged['QString'].connect(ContestDialog.clubChanged) # type: ignore
        self.specificLineEdit.textChanged['QString'].connect(ContestDialog.specificChanged) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(ContestDialog)

    def retranslateUi(self, ContestDialog):
        _translate = QtCore.QCoreApplication.translate
        ContestDialog.setWindowTitle(_translate("ContestDialog", "Contest Export"))
        self.label.setText(_translate("ContestDialog", "Contest"))
        self.label_2.setText(_translate("ContestDialog", "Band"))
        self.label_3.setText(_translate("ContestDialog", "Date from"))
        self.label_4.setText(_translate("ContestDialog", "Date to"))
        self.fromDateEdit.setDisplayFormat(_translate("ContestDialog", "yyyy-MM-dd"))
        self.toDateEdit.setDisplayFormat(_translate("ContestDialog", "yyyy-MM-dd"))
        self.label_6.setText(_translate("ContestDialog", "Callsign"))
        self.label_7.setText(_translate("ContestDialog", "Fullname"))
        self.label_5.setText(_translate("ContestDialog", "Street"))
        self.specificLabel.setText(_translate("ContestDialog", "Specific"))
        self.label_9.setText(_translate("ContestDialog", "Club"))
        self.label_10.setText(_translate("ContestDialog", "eMail"))
        self.label_11.setText(_translate("ContestDialog", "Locator"))
        self.label_13.setText(_translate("ContestDialog", "Power"))
        self.label_12.setText(_translate("ContestDialog", "Mode"))
        self.callLineEdit.setPlaceholderText(_translate("ContestDialog", "DX1XX"))
        self.locatorLineEdit.setPlaceholderText(_translate("ContestDialog", "AA11aa"))
        self.nameLineEdit.setPlaceholderText(_translate("ContestDialog", "Max Example"))
        self.emailLineEdit.setPlaceholderText(_translate("ContestDialog", "dx1xx@darc.de"))
        self.specificLineEdit.setPlaceholderText(_translate("ContestDialog", "Contest specific information"))
        self.clubLineEdit.setPlaceholderText(_translate("ContestDialog", "DARC"))
        self.streetLineEdit.setPlaceholderText(_translate("ContestDialog", "Street 1"))
        self.label_8.setText(_translate("ContestDialog", "City"))
        self.cityLineEdit.setPlaceholderText(_translate("ContestDialog", "11111 Berlin"))
        self.label_14.setText(_translate("ContestDialog", "Soapbox"))
        self.soapPlainTextEdit.setPlaceholderText(_translate("ContestDialog", "Extra information"))
        self.exportPathLineEdit.setPlaceholderText(_translate("ContestDialog", "Export path"))
        self.choosePathToolButton.setText(_translate("ContestDialog", "..."))
        self.exportPushButton.setText(_translate("ContestDialog", "Export"))
        self.cancelPushButton.setText(_translate("ContestDialog", "Cancel"))
